// A basic everyday NeoPixel strip test program.
// Arduino Timezone Library Copyright (C) 2018 by Jack Christensen and
// licensed under GNU GPL v3.0, https://www.gnu.org/licenses/gpl.html
// Jack Christensen Aug 2012

/*
March 19, 2020 
MTS BlueLamp
using WS2812B 12 LED strip
RTC DS3231 
RTC set to UTC
Using Timezone to control DST
*/

#include <Wire.h>
#include <Adafruit_NeoPixel.h>
#include "TimeLib.h" //PaulStoffregen Time Master library
#include <DS1307RTC.h>   // https://github.com/PaulStoffregen/DS1307RTC
#include <Timezone.h>    // https://github.com/JChristensen/Timezone

// US Mountain Time Zone (Denver)
TimeChangeRule MDT = { "MDT", Second, Sun, Mar, 2, -360 };    //Daylight time = UTC - 6 hours
TimeChangeRule MST = { "MST", First, Sun, Nov, 2, -420 };     //Standard time = UTC - 7 hours
Timezone myTZ(MDT, MST);

TimeChangeRule *tcr;        //pointer to the time change rule, use to get TZ abbrev

#define IntensityPin A0 //Intensity pot

// Defines value sent to each color
int RVal, GVal, BVal;
int MaxV;

bool update = true; // only update LEDs when there is a change, reduces blinking
int PrevMax;

#define LED_PIN    6  // Data pin for NeoPixels
#define LED_COUNT 12  // # of NeoPixels

#define TPin 7  // Toggle On/Off
boolean ToggleOn = false;

enum timeState {
	DAY_TIME,	//after 5a.m.
	AFTER_6P,
	AFTER_7P,
	AFTER_8P,
	AFTER_9P,
	AFTER_10P,
	AFTER_11P
};
enum timeState CurrTime;

// Declare NeoPixel strip object:
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);
// strip(# pixels, pin#,Pixel type flags)
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)


void setup(){
  pinMode(TPin, INPUT_PULLUP);
  
  Serial.begin(115200);
//while (!Serial); // M4
  setSyncProvider(RTC.get);   // the function to get the time from the RTC
  if(timeStatus()!= timeSet)
      Serial.println("Unable to sync with the RTC");
  else
      Serial.println("RTC has set the system time");
  
  strip.begin();           // INITIALIZE NeoPixel
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(255); // Set BRIGHTNESS
}

void loop(){
	time_t utc = now();
	time_t local = myTZ.toLocal(utc, &tcr); // This is local time with DST 

Serial.print(hour(local));
Serial.print(":");
Serial.println(minute(local));

  if( hour(local)  >= 5 && hour(local) < 18) CurrTime = DAY_TIME;
  if( hour(local) >= 18 && hour(local) < 19) CurrTime = AFTER_6P;
  if( hour(local) >= 19 && hour(local) < 20) CurrTime = AFTER_7P;
  if( hour(local) >= 20 && hour(local) < 21) CurrTime = AFTER_8P;
  if( hour(local) >= 21 && hour(local) < 22) CurrTime = AFTER_9P;
  if( hour(local) >= 22 || hour(local) < 5) CurrTime = AFTER_10P;
  

  // Read Intensity pot and set MaxV
  MaxV = (int)analogRead(IntensityPin)/4;
  if ( (MaxV > PrevMax ++) || (MaxV < PrevMax--)) update = true;
  PrevMax = MaxV;

  if (CurrTime == DAY_TIME){  // Max RGB
  	RVal = MaxV;
  	GVal = MaxV;
  	BVal = MaxV;
    update = true;
Serial.println("In DAY_TIME loop");
  }
  if (CurrTime == AFTER_6P){  // Turn off Blue
  	RVal = MaxV;
  	GVal = MaxV;
  	BVal = 0;
    update = true;
Serial.println("In AFTER_6P loop");
  }
  if (CurrTime == AFTER_7P){  // Reduce Green and some Red
  	RVal = MaxV * 0.8;
  	GVal = MaxV * 0.6;
  	BVal = 0;
    update = true;
  Serial.println("In AFTER_7P loop");
  }
  if (CurrTime == AFTER_8P){  // Reduce Green and some Red
  	RVal = MaxV * 0.6;
  	GVal = MaxV * 0.4;
  	BVal = 0;
    update = true;
Serial.println("In AFTER_8P loop");
  }
  if (CurrTime == AFTER_9P){  // Turn off Green and some Red
  	RVal = MaxV * 0.4;
  	GVal = 0;
  	BVal = 0;
    update = true;
  Serial.println("In AFTER_9P loop");
  }
 if (CurrTime == AFTER_10P){  // Turn off all
   	RVal = 0;
  	GVal = 0;
  	BVal = 0;
    update = true;
Serial.println("In ToggleOff loop");
 }

  // MTS so if digitalRead & ToggleOn are equal than ToggleOn will flip
  // and next time digitalRead will have to be opposite state
  // May have to press switch twice first time it is run 
  if( digitalRead(TPin) == ToggleOn ){
     ToggleOn = !ToggleOn;
  }

  // if update && ToggleOn then turn on LEDs, otherwise turn off 
  if (update && ToggleOn){
    colorWipe( strip.Color(RVal, GVal, BVal), 0 ); // update WS2812B LEDs
    update = false;
  }
  else {      // Turn off WS2812B
    colorWipe( strip.Color(0, 0, 0), 0 ); 
    update = false;
  }
  delay(1000);

}


// Some functions of our own for creating animated effects -----------------

// Fill strip pixels one after another with a color. Strip is NOT cleared
// first; anything there will be covered pixel by pixel. Pass in color
// (as a single 'packed' 32-bit value, which you can get by calling
// strip.Color(red, green, blue) as shown in the loop() function above),
// and a delay time (in milliseconds) between pixels.
void colorWipe(uint32_t color, int wait) {
  for(int i=0; i<strip.numPixels(); i++) { // For each pixel in strip...
    strip.setPixelColor(i, color);         //  Set pixel's color (in RAM)
    strip.show();                          //  Update strip to match
    delay(wait);                           //  Pause for a moment
  }
}
